@@ -8,7 +8,7 @@ import collections
 import logging
 import re
 
-from ptk.lexer import ProgressiveLexer, EOF, token
+from ptk.lexer import ProgressiveLexer, EOF, token, LexerPosition
 from ptk.grammar import Grammar, Production, GrammarError
 # production is only imported so that client code doesn't have to import it from grammar
 from ptk.grammar import production # pylint: disable=W0611
@@ -21,14 +21,16 @@ class ParseError(Exception):
 
     :ivar token: The unexpected token.
     """
-    def __init__(self, grammar, tok, state):
+    def __init__(self, grammar, tok, state, tokens):
         self.token = tok
         super().__init__('Unexpected token "%s" (%s) in state "%s"' % (tok.value, tok.type, sorted(state)))
 
+        self._state = state
         self._expecting = set()
         for terminal in grammar.tokenTypes():
             if grammar.__actions__.get((state, terminal), None) is not None:
                 self._expecting.add(terminal)
+        self._tokens = tokens
 
     def expecting(self):
         """
@@ -36,6 +38,24 @@ class ParseError(Exception):
         """
         return self._expecting
 
+    def state(self):
+        """
+        Returns the parser state when the error was encountered.
+        """
+        return self._state
+
+    def lastToken(self):
+        """
+        Returns the last valid token seen before this error
+        """
+        return self._tokens[-1]
+
+    def tokens(self):
+        """
+        Returns all tokens seen
+        """
+        return self._tokens
+
 
 def leftAssoc(*operators):
     """
@@ -110,6 +130,17 @@ class _Item(object):
         """
         return _Item(self.production, self.dot + 1, self.terminal)
 
+    def hasPrefix(self, *tokens):
+        """
+        Return True if the passed sequence of token types appears right before the dot
+        """
+        if len(tokens) > self.dot:
+            return False
+        for tok1, tok2 in zip(self.production.right[self.dot - len(tokens):self.dot], tokens):
+            if tok1 != tok2:
+                return False
+        return True
+
     def __repr__(self):
         symbols = list(self.production.right)
         symbols.insert(self.dot, '\u2022')
@@ -131,7 +162,7 @@ class _Accept(BaseException):
         super().__init__()
 
 
-_StackItem = collections.namedtuple('_StackItem', ['state', 'value'])
+_StackItem = collections.namedtuple('_StackItem', ['state', 'value', 'position'])
 
 
 class _Shift(object):
@@ -139,7 +170,7 @@ class _Shift(object):
         self.newState = newState
 
     def doAction(self, grammar, stack, tok): # pylint: disable=W0613
-        stack.append(_StackItem(self.newState, tok.value))
+        stack.append(_StackItem(self.newState, tok.value, tok.position))
         return True
 
 
@@ -149,20 +180,22 @@ class _Reduce(object):
         self.nargs = len(item.production.right)
 
     def doAction(self, grammar, stack, tok): # pylint: disable=W0613
-        callback, kwargs = self._getCallback(stack)
-        self._applied(grammar, stack, callback(grammar, **kwargs))
+        pos, (callback, kwargs) = self._getCallback(stack)
+        self._applied(grammar, stack, callback(grammar, **kwargs), pos)
         return False
 
-    def _applied(self, grammar, stack, prodVal):
-        stack.append(_StackItem(grammar.goto(stack[-1].state, self.item.production.name), prodVal))
+    def _applied(self, grammar, stack, prodVal, position):
+        stack.append(_StackItem(grammar.goto(stack[-1].state, self.item.production.name), prodVal, position))
 
     def _getCallback(self, stack):
         if self.nargs:
             args = [stackItem.value for stackItem in stack[-self.nargs:]]
+            pos = stack[-self.nargs].position
             stack[-self.nargs:] = []
         else:
             args = []
-        return self.item.production.apply(args)
+            pos = stack[-1].position # Hum.
+        return pos, self.item.production.apply(args, pos)
 
 
 class LRParser(Grammar):
@@ -190,6 +223,7 @@ class LRParser(Grammar):
             for action, stack in self._processToken(tok):
                 if action.doAction(self, stack, tok):
                     break
+            self.__tokens.append(tok)
         except _Accept as exc:
             self._restartParser()
             return self.newSentence(exc.result)
@@ -202,7 +236,7 @@ class LRParser(Grammar):
         while True:
             action = self.__actions__.get((self.__stack[-1].state, tok.type), None)
             if action is None:
-                raise ParseError(self, tok, self.__stack[-1].state)
+                raise ParseError(self, tok, self.__stack[-1].state, self.__tokens)
             yield action, self.__stack
 
     def newSentence(self, sentence): # pragma: no cover
@@ -397,7 +431,9 @@ class LRParser(Grammar):
         return cls._goto[(state, symbol)]
 
     def _restartParser(self):
-        self.__stack = [_StackItem(self._startState, None)]
+        self.__stack = [_StackItem(self._startState, None, LexerPosition(1, 1))]
+        self.__tokens = []
+        self.restartLexer()
 
     @classmethod
     @memoize
@@ -438,11 +474,24 @@ class ProductionParser(LRParser, ProgressiveLexer): # pylint: disable=R0904
 
         # DECL -> identifier "->" PRODS
         prod = Production('DECL', cls.DECL)
-        prod.addSymbol('identifier', 'name')
+        prod.addSymbol('LEFT', 'left')
         prod.addSymbol('arrow')
         prod.addSymbol('PRODS', 'prods')
         cls.__productions__.append(prod)
 
+        # LEFT -> identifier
+        prod = Production('LEFT', cls.LEFT)
+        prod.addSymbol('identifier', 'name')
+        cls.__productions__.append(prod)
+
+        # LEFT -> identifier "<" posarg ">"
+        prod = Production('LEFT', cls.LEFT)
+        prod.addSymbol('identifier', 'name')
+        prod.addSymbol('lchev')
+        prod.addSymbol('identifier', 'posarg')
+        prod.addSymbol('rchev')
+        cls.__productions__.append(prod)
+
         # PRODS -> P
         prod = Production('PRODS', cls.PRODS1)
         prod.addSymbol('P', 'prodlist')
@@ -522,10 +571,11 @@ class ProductionParser(LRParser, ProgressiveLexer): # pylint: disable=R0904
         super().prepare(**kwargs)
 
     def newSentence(self, startSymbol):
-        name, prods = startSymbol
+        (name, posarg), prods = startSymbol
         for prod in prods:
             if prod.name is None:
                 prod.name = name
+                prod.posarg = posarg
         self.grammarClass.__productions__.extend(prods)
 
     # Lexer
@@ -588,10 +638,14 @@ class ProductionParser(LRParser, ProgressiveLexer): # pylint: disable=R0904
 
     # Parser
 
-    def DECL(self, name, prods):
+    def DECL(self, left, prods):
+        name, posarg = left
         if name in self.grammarClass.tokenTypes():
             raise GrammarError('"%s" is a token name and cannot be used as non-terminal' % name)
-        return (name, prods)
+        return (left, prods)
+
+    def LEFT(self, name, posarg=None):
+        return (name, posarg)
 
     def PRODS1(self, prodlist):
         return prodlist
